#!/usr/bin/env ruby

class XY
  include Comparable
  attr_accessor :x, :y
  def initialize(x,y)
    @x=x
    @y=y
  end
  def <=> (other)
    self.x <=> other.x
  end
  def self.between(x,xy0,xy1)
    if ( x < [xy0.x,xy1.x].min or [xy0.x,xy1.x].max < x ) then
      puts xy0.x, x, xy1.x
      throw "extrapolation" 
    end
    throw "same" if ((xy1.x-xy0.x).abs<1.0e-15)
    s = (x-xy0.x)/(xy1.x-xy0.x)
    XY.new(x,s*xy1.y+(1-s)*xy0.y)
  end
end

class PLT < Array
  def parse(filename,zone=1,yindex=3)
    lines=IO.readlines(filename)
    z=0
    lines.each_with_index do |line,line_number|
      next if (line =~ /#/)
      if (line =~ /ZONE /) then
        # puts line_number.to_s+" "+line
        z+=1;
        next
      end
      next unless (z == zone)
      break if ( z>zone)
      col=line.split
      x = col[0]
      y = col[yindex]
      next unless (x && y)
      x = x.to_f
      y = y.to_f
      next if ( line =~ /[a-df-zA-DF-Z]/ ) 
      self << XY.new(x,y)
    end
    # puts "read #{length} data points from #{filename}"
    self
  end
  def export(filename)
    File.open(filename,'w') do |f|
      self.each do |xy|
        f.printf("%.15e %.15e\n",xy.x,xy.y)
      end
    end
  end
  def interpolate_to(other)
    iother=0
    iself=0
    common=Array.new
    while ( iself < self.length && iother < other.length )
      if ( self[iself] < other[iother] ) then
        common << self[iself]
        iself += 1
      else
        if ( 0 == iself ) then
          common << XY.new(other[iother].x,0.0)
        else
          common << XY.between(other[iother].x,self[iself-1],self[iself])
        end
        iother += 1
      end
    end
    PLT.new.replace(common)
  end
  def difference_to(other,x0,x1)
    a = self.interpolate_to(other)
    b = other.interpolate_to(self)
    difference = 0.0
    #first triangle
    i0 = 0
    i0 += 1 until a[i0].x>x0
    difference += 0.5*( a[i0].x-x0 )*(b[i0].x-a[i0].x).abs
    #last triangle
    i1 = a.length-1
    i1 -= 1 until a[i1].x<x1
    difference += 0.5*( x1-a[i1].x )*(b[i1].y-a[i1].y).abs
    (i0..(i1-1)).each do |i|
      difference += 
        0.5*( a[i+1].x-a[i].x ) * ((b[i].y-a[i].y).abs+(b[i+1].y-a[i+1].y).abs)
    end
    difference/(x1-x0)
  end
end

if (__FILE__ == $0) then

  target_file = ARGV[0] || "../Park_FUN3D_VLVAF_M080s_SEEB.plt"
  truth_file  = ARGV[1] || "../shiftavg_195_219-221.39.plt"
  zone        = ARGV[2] || 1
  zone = zone.to_i
  int_start = ARGV[3] || 25.0
  int_start = int_start.to_f
  int_end   = ARGV[4] || 46.0
  int_end   = int_end.to_f

  target = PLT.new.parse(target_file,zone)
  truth  = PLT.new.parse(truth_file,1)

  if (false)
    super_target = target.interpolate_to(truth)
    puts "join target length #{super_target.length}"

    super_truth = truth.interpolate_to(target)
    puts "join truth length #{super_truth.length}"

    super_target.export('target_super.plt')
    super_truth.export('truth_super.plt')
    target.export('target_orig.plt')
    truth.export('truth_orig.plt')
  end

  puts truth.difference_to(target,int_start,int_end)

end

